''' 
 *
 *  Copyright (C) 2013  André Luís da Costa, FEEC, Unicamp
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
'''

from PySide.QtCore import *
from PySide.QtGui import *
from model import *
from view import *
import pickle
import threading
from nibabel import nifti1
import sys
import numpy as np

dcm2nifti_conversion_string = 'dcm2nii -d n -e n -f y -i n -p n -g y -r n {filename}.dcm'

class Chieftain(QObject):
    __instance = None
    _currentChanged = Signal()

    @staticmethod
    def get():
        if Chieftain.__instance is None:
            Chieftain.__instance = Chieftain()
        return Chieftain.__instance

    def __init__(self):
        QObject.__init__(self)
        self._main = None
        self._datafilename = ''
        self.closeCurrent()

    def new(self):
        pass

    def open(self,  path=''):
        self._datafilename, ok = QFileDialog.getOpenFileName(None, self.tr("Load volumetric image"), "./", self.tr("NifTi Image (*.nii *.nii.gz)"))
        if ok:
            #try:
            self._current = Volume(nifti1.load(self._datafilename))
            '''except:
                QMessageBox.critical(None, "File error",
                        "Invalid file format.",
                        QMessageBox.Ok | QMessageBox.Default,
                        QMessageBox.NoButton)'''
            '''
            self._current.setScale(0.4)
            self._annotationDesk = F_Annotation(self._current)
            self._annotations = {}
            self._saver = threading.Timer(10, self.saveCurrent)
            self._saver.start()
            '''
            self._currentChanged.emit()

    def closeCurrent(self):
        self._current = None
        self._currentChanged.emit()

    def closeAll(self):
        sys.exit(0)

    def saveCurrent(self):
        pass
    
    def saveAll(self):
        pass

    def saveAs(self):
        pass

    def getCurrent(self):
        return self._current
	
    def zoomIn(self):
        pass
    
    def zoomOut(self):
        pass

class Orc(QObject):
    def __init__(self):
        QObject.__init__(self)
        self._dataset = ImageDataset()
        self._ui = Gui()
        self.__initSummary()
        self.__initDatasetOverview()
    
    def getDataset(self):
        return self._dataset
    
    def getUI(self):
        return self._ui.getTab()
    
    def __initSummary(self):
        summary = self._ui.getSummary()
        summary._dsName.setText(self._dataset.getName())
        summary._dsPurpose.setText(self._dataset.getPurpose())
    
    def __initDatasetOverview(self):
        dsOverview = self._ui.getDatasetOverview()
        dsOverview._updateImageList.connect(self.listImages)
        dsOverview._annotateImage.connect(self.annotateImage)
    
    def __initImageAnnotation(self, image):
        annotationDesk = F_Annotation(image)
        self._ui._tab.addTab(annotationDesk, "Image #%d"%image.getId())
    
    def addImages(self):
        files, x = QFileDialog.getOpenFileNames(None, self.tr("Open Images"), "./", self.tr("Image Files (*.png *.jpg *.jpeg *.gif *.pbm *.pgm *.ppm *.tif *.tiff *.xbm *.xpm *.bmp)"))
        for imagePath in files:
            self._dataset.addImage(imagePath)
        self._ui.getDatasetOverview().updateImageViewer()

    @Slot(QListWidget)
    def listImages(self, lWidget):
        lWidget.clear()
        for img in self._dataset._images:
            newListItem = QListWidgetItem(img.getIcon(), '%d'%(img.getId()), view=lWidget)

    @Slot(long)
    def annotateImage(self, imgId):
        self.__initImageAnnotation(self._dataset._images[imgId])

'''
def interp_trilinear(ff, pts):
	# ff - tri dimentional image
	# pts - array (3xN) of points to interpolate:
	# throws IndexError if there are indices out of range in pts
	# let's consider x the first dimension, y the second and z the third
	import numpy as np
	
	# integer indices
	ipts = np.floor(pts).astype(int)
	
	# fractional indices
	fpts = pts - ipts
	
	# workaround for the case some index equals the last valid index in the respective dimension
	fpts[0][ipts[0]>=ff.shape[0]-1] += 1
	ipts[0][ipts[0]>=ff.shape[0]-1] -= 1
	fpts[1][ipts[1]>=ff.shape[1]-1] += 1
	ipts[1][ipts[1]>=ff.shape[1]-1] -= 1
	fpts[2][ipts[2]>=ff.shape[2]-1] += 1
	ipts[2][ipts[2]>=ff.shape[2]-1] -= 1

	I = ipts.copy()
	Ix = ipts.copy()
	Ix[0] += 1
	Iy = ipts.copy()
	Iy[1] += 1
	Iz = ipts.copy()
	Iz[2] += 1
	Ixy = Ix.copy()
	Ixy[1] += 1
	Ixz = Ix.copy()
	Ixz[2] += 1
	Iyz = Iy.copy()
	Iyz[2] += 1
	Ixyz = Ixy.copy()
	Ixyz[2] += 1
	
	a = ff[I[0], I[1], I[2]]
	b = ff[Ix[0], Ix[1], Ix[2]] - a
	c = ff[Iy[0], Iy[1], Iy[2]] - a
	d = ff[Iz[0], Iz[1], Iz[2]] - a
	e = ff[Ixy[0], Ixy[1], Ixy[2]] - a - b - c
	f = ff[Ixz[0], Ixz[1], Ixz[2]] - a - b - d
	g = ff[Iyz[0], Iyz[1], Iyz[2]] - a - c - d
	h = ff[Ixyz[0], Ixyz[1], Ixyz[2]] - a - b - c - d - e - f - g
	
	return a + b*fpts[0] + c*fpts[1] + d*fpts[2] + e*fpts[0]*fpts[1] + f*fpts[0]*fpts[2] + g*fpts[1]*fpts[2] + h*fpts[0]*fpts[1]*fpts[2]

def affine(f,T,domain='SAME'):
	F = array(f)
	
	if len(f.shape)==2:
		F.shape = (1,) + f.shape
	
	max_s,max_m,max_n = np.array(F.shape) - 1
	
	if isinstance(domain, (list, tuple, np.ndarray)) and len(domain)==3:
		s,m,n = domain
	else:        
		s,m,n = F.shape
	
	z,y,x = np.indices([s,m,n])
	PtosHomogenio = np.array([ z.ravel(), y.ravel(), x.ravel(), np.ones(s*m*n) ])
	PtosHomogenioLinha = np.dot(np.linalg.inv(T), PtosHomogenio)
	
	#zlinha = PtosHomogenioLinha[0].round(0).astype(int).reshape(s,m,n)
	#ylinha = PtosHomogenioLinha[1].round(0).astype(int).reshape(s,m,n)
	#xlinha = PtosHomogenioLinha[2].round(0).astype(int).reshape(s,m,n)
			
	PtosHomogenioLinha[0] = np.maximum(0, np.minimum(max_s, PtosHomogenioLinha[0]))
	PtosHomogenioLinha[1] = np.maximum(0, np.minimum(max_m, PtosHomogenioLinha[1]))
	PtosHomogenioLinha[2] = np.maximum(0, np.minimum(max_n, PtosHomogenioLinha[2]))
	
	return np.interp_trilinear(F, PtosHomogenioLinha).reshape((s,m,n))

def inv(f):
	f = f.ravel()
	g = f.copy()
	s0 = f[0]*f[5] - f[1]*f[4];
	s1 = f[0]*f[9] - f[1]*f[8];
	s2 = f[0]*f[13] - f[1]*f[12];
	s3 = f[4]*f[9] - f[5]*f[8];
	s4 = f[4]*f[13] - f[5]*f[12];
	s5 = f[8]*f[13] - f[9]*f[12];

	c5 = f[10]*f[15] - f[11]*f[14];
	c4 = f[6]*f[15] - f[7]*f[14];
	c3 = f[6]*f[11] - f[7]*f[10];
	c2 = f[2]*f[15] - f[3]*f[14];
	c1 = f[2]*f[11] - f[3]*f[10];
	c0 = f[2]*f[7] - f[3]*f[6];

	det = s0 * c5 - s1 * c4 + s2 * c3 + s3 * c2 - s4 * c1 + s5 * c0;

	invDet = 1.0/det;

	g[0] = (f[5]*c5 - f[9]*c4 + f[13]*c3) * invDet;
	g[4] = (-f[4]*c5 + f[8]*c4 - f[12]*c3) * invDet;
	g[8] = (f[7]*s5 - f[11]*s4 + f[15]*s3) * invDet;
	g[12] = (-f[6]*s5 + f[10]*s4 - f[14]*s3) * invDet;

	g[1] = (-f[1]*c5 + f[9]*c2 - f[13]*c1) * invDet;
	g[5] = (f[0]*c5 - f[8]*c2 + f[12]*c1) * invDet;
	g[9] = (-f[3]*s5 + f[11]*s2 - f[15]*s1) * invDet;
	g[13] = (f[2]*s5 - f[10]*s2 + f[14]*s1) * invDet;

	g[2] = (f[1]*c4 - f[5]*c2 + f[13]*c0) * invDet;
	g[6] = (-f[0]*c4 + f[4]*c2 - f[12]*c0) * invDet;
	g[10] = (f[3]*s4 - f[7]*s2 + f[15]*s0) * invDet;
	g[14] = (-f[2]*s4 + f[6]*s2 - f[14]*s0) * invDet;

	g[3] = (-f[1]*c3 + f[5]*c1 - f[9]*c0) * invDet;
	g[7] = (f[0]*c3 - f[4]*c1 + f[8]*c0) * invDet;
	g[11] = (-f[3]*s3 + f[7]*s1 - f[11]*s0) * invDet;
	g[15] = (f[2]*s3 - f[6]*s1 + f[10]*s0) * invDet;
	return g.reshape((4,4))
'''
