#version 130
//http://www.opengl.org/wiki/GLSL_:_common_mistakes
/*  
 *  raycasting.vert: fragment shader for ray-casting rendering
 *
 *  Copyright (C) 2013  André Luís da Costa, Wu Shin-Ting, FEEC, Unicamp
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
uniform mat4 viewMatrixInv; //matriz inversa de MODELVIEW
uniform vec3 ray_direction;  
uniform vec3 step;
uniform vec3 bgColor;
uniform sampler3D VOLUME;   //volume 3D com as intensidades
uniform sampler1D OPACITY;  //volume 1D com as opacidades
uniform sampler1D COLOR;    //volume 1D com as falsas cores
in vec3 texCoord;           //coordenadas de textura

void main(void){
	 // Calcula o valor limite de t
	 vec3 velocity = abs(ray_direction);
	 vec3 lim = max((ray_direction / velocity), 0);
	 vec3 distance = abs(lim-texCoord);
	 vec3 t_xyz = distance/velocity;
	 float tmax = min(min(t_xyz.x, t_xyz.y), min(t_xyz.x, t_xyz.z));
	 
     float intensity, alpha = 0, s;
     if (step.z==1) s = step.x; else s = step.y;
	 float opacity;
	 vec3 color = bgColor, texpos, tmp_color;
	 for (float t=tmax; t>0; t-=s){
		 texpos = texCoord+ray_direction*t;
		 intensity = texture(VOLUME, texpos.xyz).r; // ler a densidade
		 opacity = texture(OPACITY, intensity).r*s*100;
		 tmp_color = texture(COLOR, intensity).rgb;
		 color = mix(tmp_color, color, (1-opacity));
	 }
	 
     gl_FragColor = vec4(color, 1);
}
